@@ -185,7 +185,8 @@ sub _snmp_connect_generic {
     $snmp_args{ $k } = setting('net_snmp_options')->{ $k };
   }
 
-  if (scalar keys %{ setting('net_snmp_options') }) {
+  if (scalar keys %{ setting('net_snmp_options') }
+      or not $snmp_args{BulkWalk}) {
     foreach my $k (sort keys %snmp_args) {
         next if $k eq 'MibDirs';
         debug sprintf 'snmp transport conf: %s => %s', $k, $snmp_args{ $k };
@@ -208,28 +209,59 @@ sub _snmp_connect_generic {
       unshift @classes, $device->snmp_class;
   }
 
-  my $info = undef;
-  COMMUNITY: foreach my $comm (@communities) {
-      next unless $comm;
+  # first try the communities in a fast pass using best version
 
-      VERSION: foreach my $ver (@versions) {
-          next unless $ver;
+  VERSION: foreach my $ver (3, 2) {
+      my %local_args = (%snmp_args,
+        Version => $ver, Retries => 0, Timeout => 200000);
+
+      COMMUNITY: foreach my $comm (@communities) {
+          next unless $comm;
 
           next if $ver eq 3 and exists $comm->{community};
           next if $ver ne 3 and !exists $comm->{community};
 
-          CLASS: foreach my $class (@classes) {
-              next unless $class;
+          my $info = _try_connect($device, $classes[0], $comm, $mode, \%local_args,
+            ($useclass ? 0 : 1) );
 
-              my %local_args = (%snmp_args, Version => $ver);
-              $info = _try_connect($device, $class, $comm, $mode, \%local_args,
-                ($useclass ? 0 : 1) );
-              last COMMUNITY if $info;
+          # if successful, restore the default/user timeouts and return
+          if ($info) {
+              my $class = $info->device_type;
+              return $class->new(
+                %snmp_args, Version => $ver,
+                _mk_info_commargs($comm),
+              );
           }
       }
   }
 
-  return $info;
+  # then revert to conservative settings and repeat with all versions
+
+  # unless user wants just the fast connections for bulk discovery
+  # or we are on the first discovery attempt of a new device
+  return unless setting('snmp_try_slow_connect');
+
+  CLASS: foreach my $class (@classes) {
+      next unless $class;
+
+      VERSION: foreach my $ver (@versions) {
+          next unless $ver;
+          my %local_args = (%snmp_args, Version => $ver);
+
+          COMMUNITY: foreach my $comm (@communities) {
+              next unless $comm;
+
+              next if $ver eq 3 and exists $comm->{community};
+              next if $ver ne 3 and !exists $comm->{community};
+
+              my $info = _try_connect($device, $class, $comm, $mode, \%local_args,
+                ($useclass ? 0 : 1) );
+              return $info if $info;
+          }
+      }
+  }
+
+  return undef;
 }
 
 sub _try_connect {
@@ -246,9 +278,10 @@ sub _try_connect {
 
   try {
       debug
-        sprintf '[%s:%s] try_connect with ver: %s, class: %s, comm: %s',
+        sprintf '[%s:%s] try_connect with v: %s, t: %s, r: %s, class: %s, comm: %s',
           $snmp_args->{DestHost}, $snmp_args->{RemotePort},
-          $snmp_args->{Version}, $class, $debug_comm;
+          $snmp_args->{Version}, ($snmp_args->{Timeout} / 1000000), $snmp_args->{Retries},
+          $class, $debug_comm;
       Module::Load::load $class;
 
       $info = $class->new(%$snmp_args, %comm_args) or return;
@@ -259,7 +292,7 @@ sub _try_connect {
       if ($reclass and $info and $info->device_type ne $class) {
           $class = $info->device_type;
           debug
-            sprintf '[%s:%s] try_connect with ver: %s, new class: %s, comm: %s',
+            sprintf '[%s:%s] try_connect with v: %s, new class: %s, comm: %s',
               $snmp_args->{DestHost}, $snmp_args->{RemotePort},
               $snmp_args->{Version}, $class, $debug_comm;
 
